// Remove Studio Markers from Source Code
// by Marcia Booth - Data Access Corporation
// Created on October, 2005
//
// Modified by Allan Kim Eriksen
// This will also:
// - Right trim lines
// - Remove more studio markers.
// - Correct an error that added an extra empty line for each source file.
// - All ending blank lines are not written back to the source file.
// - Added a business process that makes it possible to change all files for a complete workspace.
//   It will traverse recursively through all directories in the workspace.
// - Removes any LOCAL statements

Use dfClient.pkg
Use Windows.pkg
Use File_dlg.Pkg
Use dirform.pkg
Use BatchDD.pkg
Use akefs.pkg

DEFERRED_VIEW Activate_oCleanMarkersView FOR ;
;
Object oCleanMarkersView is a dbView
    Set Location to 11 19
    Set Size to 200 300


    Object oGenerateBtn is a Button
        Set Label to "Generate Source without Markers"
        Set Size to 14 123
        Set Location to 90 95
        Set enabled_state to False

        Procedure OnClick
            String sDirName sFileName
            Boolean bOK
            Get value of oSourceFile to sFileName
            Get value of oDirFormPath to sDirName
            If (sDirName <> "") begin
                Send DoProcess of oBPO
                Get pbOk of oBPO to bOk
            End
            Else Get RemoveMarker sFileName to bOK
            If (bOK) Begin
                Send info_box "Parsing completed." "Remove Marker"
            End
            Else Begin
                Send info_box "Parsed File could not be created. " "Remove Marker"
            End
        End_Procedure 

    End_Object    


    Object oSourceFile is a Form
        Set Label to "Enter the name of the source file to be parsed:"
        Set Size to 13 174
        Set Location to 31 73
        Set Label_Col_Offset to 30
        Set Label_Justification_Mode to jMode_Top
        Set Prompt_Button_Mode to pb_PromptOn


        Procedure Prompt
            Integer bOpen bReadOnly
            String sFileName sDirName

            Get Show_Dialog of oOpenDialog1 to bOpen
            If bOpen Begin
                Set enabled_state of oDirFormPath to False
                Get File_Name of oOpenDialog1 to sFileName
                Set value to sFileName
                Get value of oDirFormPath to sDirName
                // check if user is opening file as read_only
                Get TickReadOnly_State of oOpenDialog1 to bReadOnly
                If bReadOnly Begin
                    Send info_box ("Files should not be opened as ReadOnly.") "Clean Markers"
                    Set enabled_state of oGenerateBtn to False
                End
                Else Begin
                    // need to check the file's attribute -- cannot be readonly
                    Set enabled_state of oGenerateBtn to True
                End
            End
            Else Begin
                Send Info_Box "You did NOT choose a file"
                Set enabled_state of oGenerateBtn to False

            End
        End_Procedure


        Procedure Onchange
            String sValue
            Get value to sValue
            If ( sValue = "" ) Begin
            Set enabled_state of oDirFormPath to True
            Set enabled_state of oGenerateBtn to False
            End
        End_Procedure
    End_Object    // oSourceFile




    Object oOpenDialog1 is a OpenDialog
        Set Dialog_Caption to "Select a source file"
        Set Filter_String to "Programs|*.src|Views|*.vw|Report Views|*.rv|Selection Lists|*.sl|Web Object|*.wo"
        Set Initial_Folder to "C:\VDF12.0\Examples\Contact\AppSrc"
    End_Object    // oOpenDialog1




    Object oDirFormPath is a DirForm
        Set Size to 14 175
        Set Location to 63 73
        Set Label_Justification_Mode to JMode_Top
        Set Label to "Enter the name of the folder structure that contains the files to be parsed:"
        Set Label_Col_Offset to 30
        Set Prompt_Button_Mode to PB_PromptOn


        Procedure Onchange
            String sDirName sFileName
            Get value of oDirFormPath to sDirName
            If sDirName Begin
                Set enabled_state of oSourceFile to False
                Set enabled_state of oGenerateBtn to True
            End
            Else Begin
                Send Info_Box "You did NOT choose a file"
                Set enabled_state of oGenerateBtn to False
                Set enabled_state of oSourceFile to True
            End

        End_Procedure
    End_Object



    Object oCheckBoxDelBak is a CheckBox
        Set Size to 14 50
        Set Location to 116 43
        Set Label to "Create backup files"
    End_Object



    Object oFormParsedExt is a Form
        Set Size to 14 139
        Set Location to 145 104
        Set value to "src;vw;sl;dg;rv;pkg;wo;dd;inc;nui;utl"
        Set Label to "Parsed extensions:"
    End_Object

    Object oBPO is a BusinessProcess

        Property Boolean pbOk


        Procedure OnProcess
            String sDirName
            Boolean bOk

            Get value of oDirFormPath to sDirName
            Get RecursiveSearchDirectoryFiles sDirName to bOK

            Set pbOk to bOk

        End_Procedure


        Function RecursiveSearchDirectoryFiles String sSourceDirname Returns Boolean
            Boolean bOk bStop bFound
            Integer iCnt iMax iTmp
            String sFile sLine sEval sExt sSourceFile
            Handle hoDir
            Move 0 to iCnt
            Get value of oSourceFile to sSourceFile
            Get Create U_cDirectory to hoDir
            Move (sSourceDirname+"\*.*") to sFile
            Get SearchSingleDirectory of hoDir sFile DIRMODE_DIRECTORIES_ONLY to iMax
            Decrement iMax
            For iCnt from 0 to iMax
                Get Searchresult of hoDir item iCnt LIST_FILENAME to sLine
                If (sLine <> "." and sLine <> "..") Begin
                    Get RecursiveSearchDirectoryFiles (sSourceDirname + "\" + sLine) to bOk
                    If (not(bOk)) Begin
                        Send destroy of hoDir
                        Function_Return False
                    End
                    Get Cancel_Check to bStop
                    If bStop Begin
                        Send destroy of hoDir
                        Function_Return False
                    End
                End
            Loop
            Get SearchSingleDirectory of hoDir sFile DIRMODE_FILES_ONLY to iMax
            Decrement iMax
            For iCnt from 0 to iMax
                Get Searchresult of hoDir item iCnt LIST_FILENAME to sLine
                Move (right(sLine,3)) to sExt
                Move (Lowercase(sExt)) to sExt
                If (Pos(".", sExt)) Move (Remove(sExt, 1, 1)) to sExt
                Get value of oFormParsedExt to sEval
                If ((Pos(sExt, sEval)) or sSourceFile) Begin
                    Send Update_status (sSourceDirname + "\" + sLine)
                    Get RemoveMarker (sSourceDirname + "\" + sLine) to bOK
                    If (not(bOk)) Begin
                        Send destroy of hoDir
                        Function_Return False
                    End
                End
                Get Cancel_Check to bStop
                If bStop Begin
                    Send destroy of hoDir
                    Function_Return False
                End
            Loop
            Send destroy of hoDir
            Move True to bOk
            Function_Return bOk
        End_Function

        Set Display_Error_State to True
        Set Process_Caption to "AB Remover"
        Set Process_Message to "Removes //AB tags, RTRIM lines og and remove LOCAL statements"
        Set Process_Title to "Code cleanup"
        Set Allow_Cancel_State to True

    End_Object


    Procedure CloseChannel Integer iChannel
        Close_Input Channel iChannel
        Send Seq_Release_Channel iChannel
    End_Procedure



    Function FindStudioMarker String sLine Returns Boolean
        Boolean bFound

        Move False to bFound

        Case Begin
            Case (sLine = "//AB-IgnoreStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-IgnoreEnd")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-StoreStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-StoreEnd")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-StoreTopStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-StoreTopEnd")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-PanelStoreTopStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-PanelStoreTopEnd")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-PanelStoreStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-PanelStoreEnd")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-ClientStoreTopStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-ClientStoreTopEnd")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-ClientStoreStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-ClientStoreEnd")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-DDOStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-DDOEnd")
                Move True to bFound
                Case Break

            Case (Left(sLine, 6) = "//AB/ ")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-MenuPackage")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-End")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-ToolbarPackage")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-ViewStart")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-ViewEnd")
                Move True to bFound
                Case Break

            Case (sLine = "//AB-StatusBarPackage")
                Move True to bFound
                Case Break

            Case (sLine = "//IDE-FileType=ftApplication")
                Move True to bFound
                Case Break


        Case End

        Function_Return bFound
    End_Function


    // RTrim and removes local statement if present.
    Function RemoveLocal String sLine Returns String
        String sTest
        Integer iPos
        Move (Rtrim(sLine)) to sLine
        Move (ltrim(sLine)) to sTest
        Move (Left(sTest, 6)) to sTest
        Move (lowercase(sTest)) to sTest
        If (sTest = "local ") Begin
            // local is the first word in the line.
            // Now find the local statement in the line and remove it.
            Move (lowercase(sLine)) to sTest
            Move (pos("local ", sTest)) to iPos
            Move (Remove(sLine, iPos, 6)) to sLine
        End
        Function_Return sLine
    End_Function


    Function ParseFile String sParsedFile Integer iInChannel Integer iOutChannel Returns Boolean
        Boolean bParseOK bMarkerFound
        String sSourceFile
        String sInLine sLine sBuffer

        Move False to bParseOK
        Move "" to sBuffer

        Get BackupFileName sParsedFile to sSourceFile
        Direct_Input Channel iInChannel (sSourceFile)
        Direct_Output Channel iOutChannel (sParsedFile)

        Repeat
            Readln Channel iInChannel sInLine
            Move (Trim(sInLine)) to sLine
            If (not(SeqEof)) Begin

                If (sLine = "") Begin
                    // Writeln sInLine
                    Move (sBuffer + (Character(13)+character(10))) to sBuffer
                End
                Else Begin
                    Get FindStudioMarker sLine to bMarkerFound

                    If (not(bMarkerFound)) Begin
                        If (sBuffer <> "") Write sBuffer
                        Get RemoveLocal sInLine to sInLine
                        Writeln sInLine
                        Move "" to sBuffer
                    End
                End
            End

        Until (Seqeof)

        If (not (err)) Begin
            Move True to bParseOK
        End


        Function_Return bParseOK
    End_Function



    Function AvailableChannel Returns Integer
        Integer iChannelAvailable

        Get Seq_New_Channel to iChannelAvailable

        Function_Return iChannelAvailable
    End_Function



    Function BackupFileName String sFile Returns String
        String sBkupFile
        Boolean bFileExists

        Move (sFile - ".BAK") to sBkupFile

        Function_Return sBkupFile

    End_Function



    Function SaveFileBackup String sFileName Returns Boolean
        Boolean bFileExists bOK
        String sBKFile
        Integer iRetval
        Move False to bOK
        Get BackupFileName sFileName to sBKFile
        Get FileCopy sFileName sBKFile to iRetVal
        Get FileExists sBKFile to bOK
        Function_Return bOK
    End_Function



    Function RemoveMarker String sFileName Returns Boolean
        Boolean bSuccess bChecked
        Integer iInChannel iOutChannel iGarbage
        String sBKFile
        // copy file to <filename>.BAK
        Get SaveFileBackup sFileName to bSuccess

        If (bSuccess) Begin
            // open available channels
            Get AvailableChannel to iInChannel
            Get AvailableChannel to iOutChannel

            If ((iInChannel <> DF_SEQ_CHANNEL_NOT_AVAILABLE) and (iOutChannel <> DF_SEQ_CHANNEL_NOT_AVAILABLE)) Begin
                // parse file
                Get ParseFile sFileName iInChannel iOutChannel to bSuccess
                // close channels
                Send CloseChannel iInChannel
                Send CloseChannel iOutChannel
            End
        End
        Get checked_state of oCheckBoxDelBak to bChecked
        If (not (bChecked)) Begin
            Get BackupFileName sFileName to sBKFile
            Get FileDelete sBKFile to iGarbage
            // Move (SEQ_EraseFile(sBKFile)) to iGarbage
        End
        Function_Return bSuccess
    End_Function


CD_End_Object    // oCleanMarkersView
